
"""a really cheesy example of a module"""

class Bakery(object):
    def bake_pumpkin_pie(self):
        self.prepare_ingredients()
        
    def prepare_ingredients(self):
        self.prepare_eggs()
    
    def prepare_eggs(self):
        self.mix_eggs()
    
    def mix_eggs(self):
        raise ValueError("damn, didn't buy enough eggs!")



"""A plugin to run nosetests more interactively

nosetty is a plugin for [http://somethingaboutorange.com/mrl/projects/nose/ nose], a test runner for python.  It accepts various commands at the terminal, giving you some one-on-one quality time with your tracebacks.  Most importantly, editing a failure point is as easy as typing a number.  How about a screenshot?

http://farmdev.com/projects/nosetty/nosetty-screenshot.png

== Install ==

{{{
easy_install nosetty
}}}

get the [http://peak.telecommunity.com/DevCenter/EasyInstall easy_install command here].

Development versions are available as:

{{{
easy_install nosetty==dev
}}}

...or via subversion at [http://nosetty.googlecode.com/svn/trunk/#egg=nosetty-dev http://nosetty.googlecode.com/svn/trunk/]

== Usage ==

Activate the plugin like so:

{{{
nosetests --tty
}}}

But to get some useful results, you'll have to tell it how to hook into your editor and other things.  All this is described in detail on the [Recipes] page.

To give the plugin a whirl, you can [http://code.google.com/p/nosetty/source checkout the source] and type...

{{{
cd src/nosetty
easy_install .
nosetests -w examples --tty
}}}

...to get what's shown in the above screenshot.

== Project Page ==

If you're not already there, the nosetty project lives on [http://code.google.com/p/nosetty/ google code].  Please submit any bugs, patches, failing tests, et cetera using the [http://code.google.com/p/nosetty/issues/list Issue Tracker].

"""

__version__ = "0.4"

try:
    import nose
except ImportError:
    # there's probably a better way to do this that I can't think of right now.
    # In this situation, we have downloaded the module from setuptools but the 
    # dependencies (nose) haven't been installed yet
    pass
else:
    del nose
    from nosetty import *



import os, sys
from nose.tools import eq_, raises
from fixture import TempIO
from nosetty import reproduce_program, Session, NoseTTY, InteractiveError

def test_main_program_is_reproducable():
    eq_(reproduce_program(['nosetests', '-v']), ['nosetests', '-v'])
    nosepassthru = os.path.join(os.path.dirname(__file__), 'nosepassthru.py')
    p = reproduce_program([nosepassthru])
    eq_(p, [sys.executable, nosepassthru])
    assert p[0].lower().endswith('python'), "unexpected args: %s" % p

def test_session_maintains_state():
    tmp = TempIO()
    data_file = tmp.join(".session")
    session = Session(data_file, "foo")
    eq_(session.started_as_parent(), True)
    # now it's set...
    eq_(session.started_as_parent(), False)
    eq_(session.started_as_parent(), False)
    
    session2 = Session(data_file, "foo2")
    eq_(session2.started_as_parent(), True)
    eq_(session2.started_as_parent(), False)
    
    session.unregister_parent()
    eq_(session.started_as_parent(), True)
    eq_(session2.started_as_parent(), False)
    
    session2.unregister_parent()
    eq_(session2.started_as_parent(), True)

def test_edit_command_is_customizable():
    p = NoseTTY()
    
    p.tty_editor = "foober"
    p.tty_edit_cmd = "%(editor)s --foo-line %(lineno)s %(filename)s"
    eq_(p.compileEditCommand("foo_file.py", 1), 
        "foober --foo-line 1 foo_file.py")
        
    p.tty_editor = "foober"
    p.tty_edit_cmd = "%(editor)s --foo-line %(lineindex)s %(filename)s"
    eq_(p.compileEditCommand("foo_file.py", 1), 
        "foober --foo-line 0 foo_file.py")
    
    p.tty_editor = "foober"
    p.tty_edit_cmd = "%(editor)s --foo-line %(lineindex)s %(FARRBAR)s"
    raises(InteractiveError)(p.compileEditCommand)("filename.py", 1)



"""core nosetty components"""

import os, sys, re, gc, stat
import inspect
import signal
import time
from os import path
from nose.plugins import Plugin
from nose.result import ln
import subprocess
from warnings import warn
try:
    from pinocchio.output_save import calc_testname
except ImportError:
    calc_testname = None
from traceback import extract_tb, format_exception_only
try:
    import cPickle as pickle
except ImportError:
    import pickle
from unittest import _TextTestResult
separator1 = _TextTestResult.separator1
separator2 = _TextTestResult.separator2
RERUN_AFTER_EXIT = 247 # magic exit code

import logging
log = logging.getLogger('nose.plugins.nosetty')
    
def reproduce_program(argv=sys.argv):
    """return list of options that ran this test program."""
    argv = [a for a in argv] # shallow copy
    prog = argv[0]
    if os.path.exists(prog) and prog.endswith(".py"):
        # http://mail.python.org/pipermail
        # /python-list/2006-December/thread.html#419784
        if not (os.stat(prog)[stat.ST_MODE] & 
                                (stat.S_IXUSR|stat.S_IXGRP|stat.S_IXOTH)):
            # if not executable then it was *probably* a 
            # script fed into python (but sys.argv doesn't tell us this), 
            # so this is needed to rerun it
            argv.insert(0, sys.executable)
    return argv

class InteractiveError(RuntimeError):
    """error at the interactive prompt"""
    pass

class Session(object):
    """a nosetests command line user's session"""
    session_file = None
    identity = None
    data = set()
    def __init__(self, session_file, identity):
        self.session_file = session_file
        self.identity = identity
        if not os.path.exists(self.session_file):
            self.save()
        self.data = pickle.load(open(self.session_file, 'r'))
    
    def started_as_parent(self):
        """returns True if this is a controlling process"""
        if self.identity in self.data:
            return False
        else:
            self.register_parent()
            return True
    
    def register_parent(self):
        """registers current identity as a controlling process"""
        self.data.add(self.identity)
        self.save()
    
    def save(self):
        """save the current session"""
        data_file = open(self.session_file, 'w')
        pickle.dump(self.data, data_file)
        data_file.close()
        os.chmod(data_file.name, 0700)
    
    def unregister_parent(self):
        """unregisters the current identity as a controlling process"""
        self.data.remove(self.identity)
        self.save()

class NullStream(object):
    """a stream that outputs nothing.
    
    only a minimal stream implementation (for nose).
    """
    def write(self, msg):
        pass
    def writeln(self, msg=None):
        pass
        
class SwitchableStream(object):
    """a stream that can be deactivated 
    (stream sent nowhere) and reactivated again.
    
    only a minimal stream implementation (for nose).
    """
    def __init__(self, stream=sys.stderr):
        self.stream = stream
        self._real_stream = stream
        
    def deactivate(self):
        self.stream = NullStream()
    
    def activate(self):
        self.stream = self._real_stream
        
    def write(self, msg):
        self.stream.write(msg)
    
    def writeln(self, msg=""):
        self.stream.write("%s\n" % msg)

class NoseTTY(Plugin):
    """run nosetests more interactively"""
    name = 'tty'
    enabled = False
    tracebacks = []
    tty_editor = None
    tty_stream = None
    tty_edit_cmd = "%(editor)s +%(lineno)s %(filename)s"
    tty_session_file = "~/.nosetty"
    outputsave = None
    runtests_cmd = None
    
    def addError(self, test, err, capt):
        """Called when a test raises an uncaught exception. DO NOT return a
        value unless you want to stop other plugins from seeing that the
        test has raised an error.

        Parameters:
         * test:
           the test case
         * err:
           sys.exc_info() tuple
         * capt:
           Captured output, if any
        """
        self.enterErrorLoop(test, err, capt, "ERROR")

    def addFailure(self, test, err, capt, tb_info):
        """Called when a test fails. DO NOT return a value unless you
        want to stop other plugins from seeing that the test has failed.

        Parameters:
         * test:
           the test case
         * err:
           sys.exc_info() tuple
         * capt:
           Captured output, if any
         * tb_info:
           Introspected traceback info, if any
        """
        self.enterErrorLoop(test, err, capt, "FAIL")
    
    def add_options(self, parser, env=os.environ):
        super(NoseTTY, self).add_options(parser, env)
        
        env_opt = "NOSE_TTY"
        parser.add_option('--tty', 
                    action='store_true', 
                    dest=self.enableOpt,
                    default=env.get(env_opt),
                    help="Enable plugin %s: %s [%s]" %
                          (self.__class__.__name__, self.help(), env_opt))
                          
        env_opt = "NOSE_TTY_EDITOR"
        default = env.get(env_opt, None) # could be empty string
        if not default:
            default = os.environ.get('EDITOR')
        parser.add_option('--tty-editor', 
            metavar=env_opt,
            action='store', default=default,
            help=("editor program [%s or EDITOR] (currently: `%s`)" % (
                                                    env_opt, default)))
            
        env_opt = "NOSE_TTY_EDIT_CMD"
        default = env.get(env_opt, None) # could be empty string
        if not default:
            default = self.tty_edit_cmd
        parser.add_option('--tty-edit-cmd', 
            action='store', default=default,
            metavar=env_opt,
            help=(  "template to invoke edit command.  "
                    "[%s] (currently: `%s`)" % (env_opt, default)))
                    
        env_opt = "NOSE_TTY_SESSION_FILE"
        default = env.get(env_opt, None) # could be empty string
        if not default:
            default = self.tty_session_file
        parser.add_option('--tty-session-file', 
            action='store', default=default,
            metavar=env_opt,
            help=(  "path to session file used internally.  "
                    "[%s] (currently: %s)" % (env_opt, default)))
    
    def begin(self):
        self.in_rerun_state = False
    
    def compileEditCommand(self, filename, lineno):
        params = {
            'editor'        : self.tty_editor,
            'filename'      : filename, 
            'lineno'        : lineno,
            'lineindex'     : lineno-1
            }
        try:
            cmd = self.tty_edit_cmd % params
        except KeyError:
            raise InteractiveError(
                "Invalid edit command '%s' "
                "(available format strings: %s)" % (
                    self.tty_edit_cmd,
                    ", ".join(["%%(%s)s"%k for k in params.keys()])))
        return cmd
                    
    def configure(self, options, conf):
        super(NoseTTY, self).configure(options, conf)
        if not self.enabled:
            return
            
        if 'HOME' not in os.environ:
            raise EnvironmentError(
                "could not find HOME in environment.  are you running "
                "from a shell?")
            
        if options.tty_session_file:
            self.tty_session_file = options.tty_session_file
        self.tty_session_file = os.path.expanduser(self.tty_session_file)
        
        self.session = Session(self.tty_session_file, self.identity())
        if self.session.started_as_parent():
            log.info("running as **PARENT** (pid %s)", os.getpid())
            self.enterParentLoop()
        else:
            log.info("running as (CHILD) (pid %s)", os.getpid())
                    
        if options.tty_editor:
            self.tty_editor = options.tty_editor
        else:
            raise EnvironmentError(
                "could not locate editor (tried --tty-editor, "
                "NOSE_TTY_EDITOR, EDITOR)")
        if options.tty_edit_cmd:
            self.tty_edit_cmd = options.tty_edit_cmd
    
    def editFile(self, filename, lineno):
        cmd = self.compileEditCommand(filename, lineno)
        log.info("command: %s", cmd)
        returncode = os.system(cmd)
        if returncode != 0:
            raise RuntimeError("%s (exit: %s)" % (cmd, returncode))
    
    def editAtTb(self, tbinfo):
        params = tbinfo
        if not params['lineno']:
            # it is possible that a line number was not found:
            params['lineno'] = 1
        return self.editFile(params['filename'], params['lineno'])
    
    def enterErrorLoop(self, test, err, capt, error_prompt="ERROR"):
        if self.conf.stopOnError:
            return
            # because it doesn't actually stop (say, if there is an error during 
            # teardown of a test, there will be two errors.  I'm not sure if 
            # this is a nose bug or not.  consider this a workaround)
            
        etype, val, tb = err
        tbinfo = []
        for ext in extract_tb(tb):
            d = {
                'extracted': ext,
                'filename': ext[0],
                'lineno': ext[1]
            }
            if d['filename'][-3:] in ('pyc','pyo'):
                d['filename'] = d['filename'][:-1]
            tbinfo.append(d)
        self.tracebacks.append((etype, val, tbinfo))
        
        def print_(msg, terminator="\n"):
            self.tty_stream.write(msg + terminator)
        
        class ErrorStack(list):
            def flush(self):
                if len(self):
                    print_("error(s): ")
                    for msg in self:
                        print_(" - %s" % msg)
                self[:] = []
        errors = ErrorStack()
        
        # a lazy way to check for outputsave after all plugins have surely been 
        # loaded.  hmm.
        if self.outputsave is None:
            for plug in self.conf.plugins:
                if plug.name == "outputsave":
                    self.outputsave = plug
                    if calc_testname is None:
                        raise ValueError(
                            "calc_testname was None but outputsave plugin "
                            "was enabled.  whuh?")
                    outputsaved_as = os.path.join(
                                self.outputsave.save_directory,
                                "%s-%s.txt" % (
                                    error_prompt.lower(),
                                    calc_testname(test)))
                    break
            if not self.outputsave:
                self.outputsave = False
                outputsaved_as = None
        
        tb_position = len(self.tracebacks)-1
        while 1:
            etype, value, tbinfo = self.tracebacks[tb_position]
            print_("")
            print_(separator1)
            print_("%s: %s" % (error_prompt, 
                    (test.shortDescription() or str(test))))
            print_(separator2)
            if capt is not None and len(capt):
                print_(ln('>> begin captured stdout <<'))
                if self.outputsave:
                    print_("saved to %s" % outputsaved_as)
                else:
                    print_(capt, terminator="")
                print_(ln('>> end captured stdout <<'))
            print_("Traceback (most recent call last):")
            tblen = len(tbinfo)
            index_from_marker = {}
            marker = tblen
            for i in range(0, tblen):
                stack = tbinfo[i]
                index_from_marker[marker] = i
                stack_marker = "%s." % marker
                if marker < 10:
                    stack_marker += " "
                filename, lineno, name, line = stack['extracted']
                print_('  %s File "%s", line %d, in %s' % (
                            stack_marker, filename,lineno,name))
                if line:
                    print_('       %s' % line.strip())
                marker -= 1
                
            lines = format_exception_only(etype, value)
            for line in lines[:-1]:
                print_(line, ' ')
            print_(lines[-1], '')
                    
            print_("")
            print_('tty-editor="%s"' % self.tty_editor)
            print_('tty-edit-cmd="%s"' % self.tty_edit_cmd)
            print_("")
            print_('type a number to edit')
            print_('type <return> to continue')
            if capt and self.outputsave:
                print_('type "o" to open stdout')
            print_('type "r" to re-run all tests')
            print_("")
            errors.flush()
            print_("> ", terminator="")
            self.tty_stream.flush()
            try:
                line = raw_input().strip().upper()
                if line.isdigit():
                    num = int(line)
                    try:
                        index = index_from_marker[num]
                    except KeyError:
                        raise InteractiveError(
                                "trace number %s is non-existant" % num)
                    self.editAtTb(self.getTbInfo(
                        tb_position=tb_position, stack=index))
                    continue
                elif line=='R':
                    return self.enterRerunState(test)
                    ## the old way:
                    # sys.exit(RERUN_AFTER_EXIT)
                elif line=="O":
                    if outputsaved_as:
                        self.editFile(outputsaved_as, 1)
                    continue
                else:
                    break
            except KeyboardInterrupt:
                self.conf.stopOnError = True
                break
            except EOFError:
                break
            except InteractiveError, e:
                errors.append(e)
                continue
                
        print_("")
    
    def enterParentLoop(self):
        self.test_args = reproduce_program()
        self.prepareToRunAsParent()
        try:
            child = None
            try:
                while 1:
                    log.info("child command: %s", self.test_args)
                    child = subprocess.Popen(self.test_args)
                    exit = child.wait()
                    log.info("child exited %s", exit)
                    if exit == RERUN_AFTER_EXIT:
                        continue
                    else:
                        if exit < 0:
                            # negative exit code means proc was killed 
                            # by signal N...
                            warn("child was killed by signal %s" % (exit*-1))
                        sys.exit(exit)
            except KeyboardInterrupt:
                if child:
                    time.sleep(1)
                    try:
                        ps = subprocess.Popen(
                                ['ps', '-p', str(child.pid), '-o', 'pid'],
                                stdout=subprocess.PIPE)
                    except OSError:
                        log.exception("ps command failed")
                        pass
                    else:
                        output = ps.stdout.readlines()
                        log.debug("ps output %s", output)
                        if len(output) > 1:
                            # this happened during a postgres deadlock
                            sys.stdout.write("\n")
                            warn("PID %s is still alive" % child.pid)
                sys.exit(1)
        finally:
            self.session.unregister_parent()
    
    def enterRerunState(self, test):
        self.conf.stopOnError = True
        self.in_rerun_state = True
        self.nose_stream.deactivate()
    
    def finalize(self, result):
        if self.in_rerun_state:
            sys.exit(RERUN_AFTER_EXIT)
    
    def getTbInfo(self, tracebacks=None, tb_position=-1, stack=-1):
        if not tracebacks:
            tracebacks = self.tracebacks
        etype, val, tbinfo = tracebacks[tb_position]
        return tbinfo[stack]
    
    def identity(self):
        return "%s:%s" % (os.getcwd(), self.runtests_cmd)
    
    def prepareToRunAsParent(self):
        # all the parent does is run a loop around the child, then exit.  So 
        # this is some kind of attempt to cut down on its footprint
        
        ## I think this is a little too dangerous (atexit functions especially)
        
        # local_objects = set()
        # for o in globals().values():
        #     try:
        #         # chincy way to get module names...
        #         local_objects.add(o.__name__)
        #         local_objects.add(o.__module__)
        #     except AttributeError:
        #         pass
        # local_objects.add(__name__)
        # 
        # for modname in sys.modules.keys():
        #     if modname not in local_objects:
        #         del sys.modules[modname]
        
        gc.collect()
                
    def setOutputStream(self, stream):
        # copy the stream...
        self.tty_stream = stream        
        self.nose_stream = SwitchableStream()
        return self.nose_stream
    



from setuptools import setup, find_packages
import pydoc, inspect
import nosetty
description, long_description = pydoc.splitdoc(inspect.getdoc(nosetty))

setup(
    name='nosetty',
    version=nosetty.__version__,
    url="http://code.google.com/p/nosetty/",
    zip_safe=False,
    author="Kumar McMillan",
    author_email = "kumar dot mcmillan / gmail.com",
    description = description,
    long_description = long_description,
    install_requires='nose',
    license = 'GNU LGPL',
    packages = find_packages(),
    keywords = 'test unittest nose nosetests plugin',
    entry_points = {
        'nose.plugins': ['tty = nosetty:NoseTTY'],
    },
)





import os
import shutil
import subprocess
from fixture import TempIO
from nosetty import NoseTTY
from nose.tools import eq_
try:
    # 0.10.0 :
    from nose.plugins import PluginTester, NoseStream
except ImportError:
    from nosetrim.test import PluginTester, NoseStream
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

class NoseTTYTester(PluginTester):
    activate_opt = "--with-tty"
    addargs = ['--tty-editor=echo', "--tty-session-file", '~/.nosetty-test']
    env = os.environ.copy()
    env['NOSE_TTY_EDITOR'] = ""
    env['NOSE_TTY_EDIT_CMD'] = ""
    env['EDITOR'] = ""
    env['NOSE_TTY_SESSION_FILE'] = ""
    
    def inputLines(self, *lines):
        for line in lines:
            self.nose.proc.stdin.write("%s\n" % line)
            self.nose.proc.stdin.flush()
        self.nose.proc.stdin.close()
        
    def setUp(self):
        super(NoseTTYTester, self).setUp()
    
    def _makeNose(self):
        """returns a NoseStream object."""
        # print self._args
        return NoseStream( subprocess.Popen(self._args, 
                                    stdout=subprocess.PIPE, 
                                    stderr=subprocess.STDOUT,
                                    stdin=subprocess.PIPE,
                                    cwd=self.suitepath, env=self.env, bufsize=1,
                                    universal_newlines=True))

class WithTestMethod(object):
    """mixin to run tests with a failing method"""
    def assert_failure_edited(self):
        self.inputLines("1")
        assert "+3" in self.nose
        assert "test_to_fail.py" in self.nose
        
    def makeSuite(self):
        self.tmp = TempIO()
        self.tmp.putfile("test_to_fail.py",
            """\
def test_bad():
    print "it's no good!!!"
    assert False""")
        return self.tmp

class WithUnitTest(object):
    """mixin to run tests with a failing unittest.TestCase"""
    def assert_failure_edited(self):
        self.inputLines("1")
        assert "+4" in self.nose
        assert "test_to_fail_unittest.py" in self.nose
        
    def makeSuite(self):
        self.tmp = TempIO()
        self.tmp.putfile("test_to_fail_unittest.py",
            """\
import unittest
class TestToFail(unittest.TestCase):
    def test_bad_from_unittest(self):
        assert False
            """)
        return self.tmp

class WithTestClass(object):
    """mixin to run tests with a failing test class"""
    def assert_failure_edited(self):
        self.inputLines("1")
        assert "+3" in self.nose
        assert "test_to_fail_testclass.py" in self.nose
        
    def makeSuite(self):
        self.tmp = TempIO()
        self.tmp.putfile("test_to_fail_testclass.py",
            """\
class TestToFail(object):
    def test_bad_from_testclass(self):
        assert False
            """)
        return self.tmp

class EditingErrorsByNumber(NoseTTYTester):
    def test_typing_a_number_edits_file_in_traceback(self):
        self.assert_failure_edited()

class TestEditingErrorsByNumberWithMethod(
        WithTestMethod, EditingErrorsByNumber):
    pass
class TestEditingErrorsByNumberWithUnittest(
        WithUnitTest, EditingErrorsByNumber):
    pass
class TestEditingErrorsByNumberWithTestClass(
        WithTestClass, EditingErrorsByNumber):
    pass

class EditErrorsByNumber(EditingErrorsByNumber):
    def makeSuite(self):
        self.tmp = TempIO()
        self.tmp.putfile("test_to_fail_nested.py",
            """\
def two():
    raise ValueError
def three():
    two()
def four():
    three()
def test_nested_fail():
    four()""")
        return self.tmp

class TestEditErrorAtNumber1(EditErrorsByNumber):        
    def assert_failure_edited(self):
        self.inputLines("1")
        assert "+2" in self.nose
        assert "test_to_fail_nested.py" in self.nose

class TestEditErrorAtNumber2(EditErrorsByNumber):        
    def assert_failure_edited(self):
        self.inputLines("2")
        assert "+4" in self.nose
        assert "test_to_fail_nested.py" in self.nose
        
class TestEditErrorAtNumber3(EditErrorsByNumber):        
    def assert_failure_edited(self):
        self.inputLines("3")
        assert "+6" in self.nose
        assert "test_to_fail_nested.py" in self.nose
        
class TestEditErrorAtNumber4(EditErrorsByNumber):        
    def assert_failure_edited(self):
        self.inputLines("4")
        assert "+8" in self.nose
        assert "test_to_fail_nested.py" in self.nose
        
class TestEditErrorAtNonexistantNumber(EditErrorsByNumber):        
    def assert_failure_edited(self):
        self.inputLines("99")
        assert "error(s):" in self.nose
        self.nose.reset_buffer()
        assert " - trace number" in self.nose
    

class TestEditorOption(WithTestMethod, NoseTTYTester):
    def test_option_can_set_editor(self):
        """--tty-editor option can set editor"""
        self.assert_failure_edited()
    
class TestEditorVar(WithTestMethod, NoseTTYTester):
    addargs = []
    env = NoseTTYTester.env
    env['NOSE_TTY_EDITOR'] = 'echo'
    
    def test_env_var_can_set_editor(self):
        """NOSE_TTY_EDITOR var can set editor"""
        self.assert_failure_edited()
    
class TestDefaultEditor(WithTestMethod, NoseTTYTester):
    addargs = []
    env = NoseTTYTester.env
    env['NOSE_TTY_EDITOR'] = ""
    env['EDITOR'] = 'echo'
    
    def test_env_var_can_set_editor(self):
        """EDITOR var can set editor"""
        self.assert_failure_edited()
    
class TestEditCommandVar(WithTestMethod, NoseTTYTester):
    addargs = ['--tty-editor=echo']
    debuglog = 'nose.plugins.nosetty'
    env = os.environ.copy()
    env['NOSE_TTY_EDIT_CMD'
                    ] = "%(editor)s %(filename)s --any-custom-opt=%(lineno)s"
    
    def assert_failure_edited(self):
        self.inputLines("1")
        assert "test_to_fail.py --any-custom-opt=3" in self.nose
    
    def test_env_var_can_set_edit_command(self):
        """NOSE_TTY_EDIT_CMD can set edit command"""
        self.assert_failure_edited()

class TestInvalidEditCommand(WithTestMethod, NoseTTYTester):    
    addargs = ['--tty-edit-cmd=%(editor)s %(noop)s'] # invalid template
    def test_invalid_edit_command_prints_error(self):
        self.inputLines("1")
        assert "error(s):" in self.nose
        self.nose.reset_buffer()
        assert " - Invalid edit command" in self.nose
        
    
class TestEditCommandOption(WithTestMethod, NoseTTYTester):
    addargs = [
        '--tty-edit-cmd="%(editor)s %(filename)s --any-custom-opt=%(lineno)s"',     
        '--tty-editor=echo']
    
    def assert_failure_edited(self):
        self.inputLines("1")
        assert "test_to_fail.py --any-custom-opt=3" in self.nose
    
    def test_tty_edit_cmd_can_set_edit_command(self):
        """--tty-edit-cmd can set edit command"""
        self.assert_failure_edited()

class TestSuccessfulRun(WithTestMethod, NoseTTYTester):
    def makeSuite(self):
        self.tmp = TempIO()
        self.tmp.putfile("test_to_pass.py",
            """def test_good(): pass""")
        return self.tmp
        
    def test_absence_of_error_does_nothing(self):
        assert "test_to_pass.py" not in self.nose

class TestRerunCommand(WithTestMethod, NoseTTYTester):
    addargs = NoseTTYTester.addargs + ['--verbose']
    # debuglog = "nose.plugins.nosetty"
    env = NoseTTYTester.env
    env['NOSE_TTY_EDITOR'] = 'echo'
    num_reruns = 4
    
    def accept_token(self, line):
        """returns true if something in the line should restart the counter
        
        i.e. you could count how many times 
        "type "r" to re-run all tests" appears
        """
        return "test_foo.test_will_fail ..." in line
        
    def makeSuite(self):
        self.tmp = TempIO()
        self.tmp.test = "test"
        self.tmp.test.putfile("__init__.py",
            """
import os
def teardown():
    f = open(os.path.join(os.path.dirname(__file__),'foo.out'), 'w')
    f.write("blah")
    f.close()
    pass
            """)
        self.tmp.test.putfile("test_foo.py",
            """
def test_will_fail():
    raise AssertionError
            """)
        return self.tmp
    
    def onrun(self, run_number):
        """called for each run number"""
        if run_number == 2:
            assert os.path.exists(self.tmp.test.join('foo.out')), (
                "expected package level teardown() to "
                "generate 'foo.out' in %s" % (
                    os.listdir(self.tmp.test)))
    
    def send_eof(self):
        self.nose.proc.stdin.close()
    
    def send_rerun_cmd(self):
        self.nose.proc.stdin.write("r\n")
        self.nose.proc.stdin.flush()
        
    def test_r_command_reruns_test_suite(self):
        ticker = 0
        run_number = 0
        ticker_limit = self.num_reruns*100
        self.send_rerun_cmd()
        for line in self.nose:
            ticker += 1
            last_run = run_number
            if self.accept_token(line):
                run_number += 1
                try:
                    self.send_rerun_cmd()
                except ValueError:
                    # IO op on closed stdin
                    pass
                if run_number == self.num_reruns:
                    self.send_eof()
            if run_number != last_run:
                try:
                    self.onrun(run_number)
                except:
                    self.send_eof()
                    raise
            if ticker > ticker_limit:
                raise RuntimeError("time out")
        returncode = self.nose.proc.wait()
        
        # expect the first run + number of re-runs...
        eq_(run_number, self.num_reruns+1)

class OutputsavePlugin(NoseTTYTester):
    addargs = [
        '--tty-edit-cmd', "%(editor)s 'OPENFILE:%(filename)s LINE:%(lineno)s'",
        '--tty-editor=echo', 
        '--with-outputsave', '--save-directory=output']
    
class TestOutputsavePlugin(WithTestMethod, OutputsavePlugin):
    def assert_failure_edited(self):
        self.inputLines("o")
        filename = os.path.join(self.tmp, "output", 
                                "fail-test_to_fail.test_bad.txt")
        assert "OPENFILE:%s LINE:1" % filename in self.nose
        assert 'type "o" to open stdout' in self.nose
    
    def test_typing_o_opens_saved_output(self):
        self.assert_failure_edited()

class TestOutputsavePluginWhenNoOutput(WithTestClass, OutputsavePlugin):
    def assert_failure_edited(self):
        self.inputLines("")
        assert 'type "o" to open stdout' not in self.nose
    
    def test_if_no_output_then_there_is_no_prompt(self):
        self.assert_failure_edited()
        